@objc(TappxPhonegap) class TappxPhonegap : CDVPlugin {
    var bannerSize: CGSize?
    var positionBanner: positionBanner = .bottom
    var tappxBanner: TappxBannerViewController?
    var tappxInterstitial: TappxInterstitialViewController?
    
    var tappxGDPR: TappxGDPR?
    
    // Callbacks
    var bannerCommand: CDVInvokedUrlCommand?
    var interstitialCommand: CDVInvokedUrlCommand?
    
    var callbackResult = CDVPluginResult(
        status: CDVCommandStatus_ERROR
    )
    
    @objc(createInterstitial:)
    
    func createInterstitial(_ command: CDVInvokedUrlCommand) {
        
        self.interstitialCommand = command
        
        // Add the Tappx App key from the parameter's function
        let tappxKey = command.arguments[1] as? String ?? ""
        TappxFramework.addTappxKey(tappxKey)
        
        // Get the Interstitial if is ready
        let interstitialReady = command.arguments[0] as? Bool ?? false
        
        tappxInterstitial = TappxInterstitialViewController(delegate: self)
        tappxInterstitial?.setAutoShowWhenReady(interstitialReady)
        
        tappxInterstitial?.load()
        
    }
    
    @objc(showInterstitial:)
    
    func showInterstitial(_ command: CDVInvokedUrlCommand) {
        self.tappxInterstitial?.show()
    }
    
    @objc(isReady:)
    func isReady(_ command: CDVInvokedUrlCommand) -> Bool {
        return (self.tappxInterstitial?.isReady)!
    }
    
    @objc(createBanner:)
    
    func createBanner(_ command: CDVInvokedUrlCommand) {
        
        self.bannerCommand = command
        
        // Get the position from the parameter's function
        let positionBannerString = command.arguments[0] as? String ?? ""
        
        if positionBannerString == "bottom" {
            positionBanner = .bottom
        } else {
            positionBanner = .top
        }
        
        // Add the Tappx App key from the parameter's function
        let tappxKey = command.arguments[1] as? String ?? ""
        TappxFramework.addTappxKey(tappxKey)
        
        // position the tappx banner into the same position as the banner view
        switch positionBanner {
        case .top:
            tappxBanner = TappxBannerViewController.init(delegate: self, andSize: TappxBannerSize.smartBanner, andPosition: .custom)
        case .bottom:
            tappxBanner = TappxBannerViewController.init(delegate: self, andSize: TappxBannerSize.smartBanner, andPosition: .bottom)
        }
        
        // load the tappx bannner
        tappxBanner?.load()
    }
    
    @objc(tappxAutoConsentGDPR:)
    
    func tappxAutoConsent(_ command: CDVInvokedUrlCommand) {
        TappxFramework.setAutoPrivacyDisclaimerEnable(true)
    }
    
    @objc(tappxAssistConsentGDPR:)
    
    func tappxAssistConsentGDPR(_ command: CDVInvokedUrlCommand) {
        tappxGDPR = TappxGDPR.init()
        tappxGDPR?.checkAndShowPrivacyDisclaime(self.getCurrentViewController())
    }
    
    @objc(tappxRenewConsentGDPR:)
    
    func tappxRenewConsentGDPR(_ command: CDVInvokedUrlCommand) {
        tappxGDPR = TappxGDPR.init()
        tappxGDPR?.renewPrivacyDisclaime(self.getCurrentViewController())
    }
    
    @objc(customTappxConsentGDPR:)
    
    func customGrantConsentGDPR(_ command: CDVInvokedUrlCommand) {
        let consent = command.arguments[0] as? Bool ?? false
        TappxFramework.acceptPersonalInfoContent(consent)
    }
    
    @objc(setCustomConsentGDPR:)
    
    func setCustomConsentGDPR(_ command: CDVInvokedUrlCommand) {
        let daisyChain = command.arguments[0] as? String ?? ""
        TappxFramework.setGDPRConsent(daisyChain)
    }
    
    func getCurrentViewController() -> UIViewController? {
        
        if let rootController = UIApplication.shared.keyWindow?.rootViewController {
            var currentController: UIViewController! = rootController
            
            while( currentController.presentedViewController != nil ) {
                currentController = currentController.presentedViewController
            }
            
            return currentController
        }
        
        return nil
    }
    
    func addBanner(positionBanner: positionBanner) {
        var phoneGapView: UIViewController = UIViewController()
        let bannerView: UIViewController = UIViewController()
        
        // Asign the phonegap view to the viewcontroller
        phoneGapView = self.viewController
        
        // Create new frames to adapt into the screen
        var phoneGapFrame = CGRect.init()
        var bannerFrame = CGRect.init()
        
        // Need to set the banner origin X by the phone because The banner will adapt only on the phoneGapView
        var getBanerOriginX: CGFloat = (UIScreen.main.bounds.width - (tappxBanner?.view.frame.size.width)!)
        if getBanerOriginX < 0 {
            getBanerOriginX = 0
        }
        
        // according the position will put the banner view on top or bottom, and will make a dynamic height
        // position the tappx banner into the same position as the banner view
        switch positionBanner {
        case .top:
            // Will Adapt programmatically the position on the bannner
            tappxBanner?.view.frame = CGRect.init(x: (getBanerOriginX / 2), y: -(self.bannerSize?.height)!, width: (tappxBanner?.view.frame.size.width)!, height: (tappxBanner?.view.frame.size.height)!)
            phoneGapFrame = CGRect.init(x: 0, y: (self.bannerSize?.height)!, width: (self.viewController?.view.frame.size.width)!, height: ((self.viewController?.view.frame.size.height)! - (self.bannerSize?.height)!))
            bannerFrame = CGRect.init(x: 0, y: -(self.bannerSize?.height)!, width: (self.viewController?.view.frame.size.height)!, height: -(self.bannerSize?.height)!)
        case .bottom:
            phoneGapFrame = CGRect.init(x: 0, y: 0, width: (self.viewController?.view.frame.size.width)!, height: ((self.viewController?.view.frame.size.height)! - (self.bannerSize?.height)!))
            bannerFrame = CGRect.init(x: 0, y: ((self.viewController?.view.frame.size.height)! - (self.bannerSize?.height)!), width: (self.viewController?.view.frame.size.width)!, height: (self.bannerSize?.height)!)
        }
        
        // Asignate the position on the view controller
        self.viewController?.view.frame = phoneGapFrame
        bannerView.view.frame = bannerFrame
        
        bannerView.view.backgroundColor = UIColor.black
        phoneGapView.view.backgroundColor = UIColor.black
        
        self.viewController?.view.addSubview(bannerView.view)
        
        // Success Banner Callback
        callbackResult = CDVPluginResult.init(status: CDVCommandStatus_OK)
        
        self.commandDelegate!.send(
            callbackResult,
            callbackId: bannerCommand!.callbackId
        )
    }
}

enum positionBanner {
    case top
    case bottom
}

extension TappxPhonegap: TappxBannerViewControllerDelegate, TappxInterstitialViewControllerDelegate {
    func presentViewController() -> UIViewController! {
        return self.getCurrentViewController()
    }
    
    // Delegates
    func tappxBannerViewControllerDidFinishLoad(_ vc: TappxBannerViewController!) {
        self.bannerSize = vc.view.frame.size
        self.addBanner(positionBanner: self.positionBanner)
        self.commandDelegate.evalJs("tappxBannerDidFinishLoad()")
    }
    func tappxBannerViewControllerDidFail(_ vc: TappxBannerViewController!, withError error: TappxErrorAd!) {
        // Error Banner Callback
        callbackResult = CDVPluginResult.init(status: CDVCommandStatus_ERROR)
        self.commandDelegate!.send(
            callbackResult,
            callbackId: bannerCommand!.callbackId
        )
        self.commandDelegate.evalJs("tappxBannerDidFail()")
    }
    func tappxBannerViewControllerDidPress(_ vc: TappxBannerViewController!) {
        self.commandDelegate.evalJs("tappxBannerDidPress()")
    }
    func tappxBannerViewControllerDidClose(_ vc: TappxBannerViewController!) {
        self.commandDelegate.evalJs("tappxBannerDidClose()")
    }
    
    func tappxInterstitialViewControllerDidFinishLoad(_ viewController: TappxInterstitialViewController!) {
        // Succces Interstitial Callback
        callbackResult = CDVPluginResult.init(status: CDVCommandStatus_OK)
        self.commandDelegate!.send(
            callbackResult,
            callbackId: interstitialCommand!.callbackId
        )
        self.commandDelegate.evalJs("tappxInterstitialDidFinishLoad()")
    }
    
    func tappxInterstitialViewControllerDidFail(viewController: TappxInterstitialViewController!, withError error: Error!) {
        // Error Interstitial Callback
        callbackResult = CDVPluginResult.init(status: CDVCommandStatus_ERROR)
        self.commandDelegate!.send(
            callbackResult,
            callbackId: interstitialCommand!.callbackId
        )
        self.commandDelegate.evalJs("tappxInterstitialDidFail()")
    }
    func tappxInterstitialViewControllerDidPress(_ viewController: TappxInterstitialViewController!) {
        self.commandDelegate.evalJs("tappxInterstitialDidPress()")
    }
    func tappxInterstitialViewControllerDidClose(_ viewController: TappxInterstitialViewController!) {
        self.commandDelegate.evalJs("tappxInterstitialDidClose()")
    }
    func tappxInterstitialViewControllerDidAppear(_ viewController: TappxInterstitialViewController!) {
        self.commandDelegate.evalJs("tappxInterstitialDidAppear()")
    }
}


